% ADVread.m
% This script opens ADV .dat files, extracts and filters the velocity and
% save the valid data.

clear all

% Settings
casename = 'TEST01201002261244.dat';
pastelist = 1;          % 1 for pasting velocity data of all files after each other
ADVlength = 4500;       % length of datasets, only required if pastelist is on
spikethresh1 = 3;       % Threshold, factor for velocity SUBSEQUENT differences (wrt mean diff)
spikethresh2 = 4;       % Threshold, factor for velocity difference (wrt STDV)
ampthresh = 3;          % Threshold, factor of amplitude (wrt std)
SNRthresh = -5;         % Threshold, minimum Signal to Noise Ratio
corrthresh = 60;        % Threshold, minimum correlation in beam velocity

% Processing data
list=dir(strcat(casename,'*.dat'));
nrfiles = length(list);
for i = 1:nrfiles
    % Opening data
    fid = fopen([casename,list(i).name],'r');
    data=fscanf(fid,'%f',[18 inf]);
    V1 = robustfilt(data(3,:),spikethresh2);
    V2 = robustfilt(data(4,:),spikethresh2);
    V3 = robustfilt(data(5,:),spikethresh2);
    V4 = robustfilt(data(6,:),spikethresh2);
    [amp1,ampind1] = robustfilt(data(7,:),ampthresh);
    [amp2,ampind2] = robustfilt(data(8,:),ampthresh);
    [amp3,ampind3] = robustfilt(data(9,:),ampthresh);
    [amp4,ampind4] = robustfilt(data(10,:),ampthresh);
    SNR1 = data(11,:);
    SNR2 = data(12,:);
    SNR3 = data(13,:);
    SNR4 = data(14,:);
    corr1 = data(15,:);
    corr2 = data(16,:);
    corr3 = data(17,:);
    corr4 = data(18,:);
    
    % Filtering data
    veldata = ADVfilter([V1; V2; V3; V4],spikethresh1);
    badind1 = [find(SNR1<SNRthresh) find(corr1<corrthresh) ampind1];
    badind2 = [find(SNR2<SNRthresh) find(corr2<corrthresh) ampind2];
    badind3 = [find(SNR3<SNRthresh) find(corr3<corrthresh) ampind3];
    badind4 = [find(SNR4<SNRthresh) find(corr4<corrthresh) ampind4];
    veldata(1,badind1) = NaN;
    veldata(2,badind2) = NaN;
    veldata(3,badind3) = NaN;
    veldata(4,badind4) = NaN;
    disp(' ')
    disp(['mean velocities: ',num2str(nanmean(veldata(1,:))),' ',num2str(nanmean(veldata(2,:))),' ',num2str(nanmean(veldata(3,:))),' ',num2str(nanmean(veldata(4,:)))]);
    disp(['mean SNR: ',num2str(nanmean(SNR1)),' ',num2str(nanmean(SNR2)),' ',num2str(nanmean(SNR3)),' ',num2str(nanmean(SNR4))]);
    disp(['mean corr: ',num2str(nanmean(corr1)),' ',num2str(nanmean(corr2)),' ',num2str(nanmean(corr3)),' ',num2str(nanmean(corr4))]);
    disp(['mean amp: ',num2str(nanmean(amp1)),' ',num2str(nanmean(amp2)),' ',num2str(nanmean(amp3)),' ',num2str(nanmean(amp4))]);
    
    save([casename,list(i).name(1:end-4),'filt.mat'],'veldata');
    if pastelist
        if i == 1
            allveldata = nan(4,ADVlength,nrfiles);
        end
        allveldata(:,1:length(veldata),i) = veldata;
    end
end

clear fid i list nrfiles *thresh data pastelist ampind* badind* spike*